@@ -3,6 +3,7 @@
 
 /*
  * Copyright (C) 2001 Havoc Pennington
+ * Copyright (C) 2005-2007 Vincent Untz
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Library General Public
@@ -850,11 +851,13 @@ _wnck_close (Screen *screen,
   xev.xclient.data.l[3] = 0;
   xev.xclient.data.l[4] = 0;
 
+  _wnck_error_trap_push ();
   XSendEvent (gdk_display,
               RootWindowOfScreen (screen),
               False,
 	      SubstructureRedirectMask | SubstructureNotifyMask,
 	      &xev); 
+  _wnck_error_trap_pop ();
 }
 
 #define _NET_WM_MOVERESIZE_SIZE_TOPLEFT      0
@@ -888,11 +891,13 @@ _wnck_keyboard_move (Screen *screen,
   xev.xclient.data.l[3] = 0; /* unused */
   xev.xclient.data.l[4] = _wnck_get_client_type ();
 
+  _wnck_error_trap_push ();
   XSendEvent (gdk_display,
               RootWindowOfScreen (screen),
               False,
               SubstructureRedirectMask | SubstructureNotifyMask,
               &xev); 
+  _wnck_error_trap_pop ();
 }
 
 void
@@ -914,11 +919,13 @@ _wnck_keyboard_size (Screen *screen,
   xev.xclient.data.l[3] = 0; /* unused */
   xev.xclient.data.l[4] = _wnck_get_client_type ();
 
+  _wnck_error_trap_push ();
   XSendEvent (gdk_display,
               RootWindowOfScreen (screen),
               False,
               SubstructureRedirectMask | SubstructureNotifyMask,
               &xev); 
+  _wnck_error_trap_pop ();
 }
 
 void
@@ -947,11 +954,13 @@ _wnck_change_state (Screen  *screen,
   xev.xclient.data.l[3] = _wnck_get_client_type ();
   xev.xclient.data.l[4] = 0;
 
+  _wnck_error_trap_push ();
   XSendEvent (gdk_display,
 	      RootWindowOfScreen (screen),
               False,
 	      SubstructureRedirectMask | SubstructureNotifyMask,
 	      &xev);
+  _wnck_error_trap_pop ();
 }
 
 void
@@ -974,11 +983,13 @@ _wnck_change_workspace (Screen     *screen,
   xev.xclient.data.l[3] = 0;
   xev.xclient.data.l[4] = 0;
 
+  _wnck_error_trap_push ();
   XSendEvent (gdk_display,
 	      RootWindowOfScreen (screen),
               False,
 	      SubstructureRedirectMask | SubstructureNotifyMask,
 	      &xev);
+  _wnck_error_trap_pop ();
 }
 
 void
@@ -1005,11 +1016,13 @@ _wnck_activate (Screen *screen,
   xev.xclient.data.l[3] = 0;
   xev.xclient.data.l[4] = 0;
 
+  _wnck_error_trap_push ();
   XSendEvent (gdk_display,
 	      RootWindowOfScreen (screen),
               False,
 	      SubstructureRedirectMask | SubstructureNotifyMask,
 	      &xev); 
+  _wnck_error_trap_pop ();
 }
 
 void
@@ -1032,11 +1045,13 @@ _wnck_activate_workspace (Screen *screen,
   xev.xclient.data.l[3] = 0;
   xev.xclient.data.l[4] = 0;
 
+  _wnck_error_trap_push ();
   XSendEvent (gdk_display,
 	      RootWindowOfScreen (screen),
               False,
 	      SubstructureRedirectMask | SubstructureNotifyMask,
 	      &xev);
+  _wnck_error_trap_pop ();
 }
 
 void
@@ -1059,11 +1074,13 @@ _wnck_change_viewport (Screen *screen,
   xev.xclient.data.l[3] = 0;
   xev.xclient.data.l[4] = 0;
 
+  _wnck_error_trap_push ();
   XSendEvent (gdk_display,
 	      RootWindowOfScreen (screen),
               False,
 	      SubstructureRedirectMask | SubstructureNotifyMask,
 	      &xev);
+  _wnck_error_trap_pop ();
 }
 
 void
@@ -1085,11 +1102,13 @@ _wnck_toggle_showing_desktop (Screen  *screen,
   xev.xclient.data.l[3] = 0;
   xev.xclient.data.l[4] = 0;
 
+  _wnck_error_trap_push ();
   XSendEvent (DisplayOfScreen (screen),
 	      RootWindowOfScreen (screen),
               False,
 	      SubstructureRedirectMask | SubstructureNotifyMask,
 	      &xev);
+  _wnck_error_trap_pop ();
 }
 
 char*
@@ -1648,7 +1667,10 @@ get_cmap (GdkPixmap *pixmap)
   if (cmap &&
       (gdk_colormap_get_visual (cmap)->depth !=
        gdk_drawable_get_depth (pixmap)))
-    cmap = NULL;
+    {
+      g_object_unref (G_OBJECT (cmap));
+      cmap = NULL;
+    }
   
   return cmap;
 }
@@ -1679,22 +1701,22 @@ _wnck_gdk_pixbuf_get_from_pixmap (GdkPixbuf   *dest,
 
   if (drawable)
     {
-			cmap = get_cmap (drawable);
+      cmap = get_cmap (drawable);
 
-			/* GDK is supposed to do this but doesn't in GTK 2.0.2,
-			 * fixed in 2.0.3
-			 */
-			if (width < 0)
-				gdk_drawable_get_size (drawable, &width, NULL);
-			if (height < 0)
-				gdk_drawable_get_size (drawable, NULL, &height);
-			
-			retval = gdk_pixbuf_get_from_drawable (dest,
-																						 drawable,
-																						 cmap,
-																						 src_x, src_y,
-																						 dest_x, dest_y,
-																						 width, height);
+      /* GDK is supposed to do this but doesn't in GTK 2.0.2,
+       * fixed in 2.0.3
+       */
+      if (width < 0)
+        gdk_drawable_get_size (drawable, &width, NULL);
+      if (height < 0)
+        gdk_drawable_get_size (drawable, NULL, &height);
+
+      retval = gdk_pixbuf_get_from_drawable (dest,
+                                             drawable,
+                                             cmap,
+                                             src_x, src_y,
+                                             dest_x, dest_y,
+                                             width, height);
     }
 
   if (cmap)
@@ -1858,7 +1880,7 @@ _wnck_icon_cache_new (void)
 {
   WnckIconCache *icon_cache;
 
-  icon_cache = g_new0 (WnckIconCache, 1);
+  icon_cache = g_slice_new0 (WnckIconCache);
 
   icon_cache->origin = USING_NO_ICON;
   icon_cache->prev_pixmap = None;
@@ -1903,7 +1925,7 @@ _wnck_icon_cache_free (WnckIconCache *icon_cache)
 {
   clear_icon_cache (icon_cache, FALSE);
   
-  g_free (icon_cache);
+  g_slice_free (WnckIconCache, icon_cache);
 }
 
 void
@@ -2318,11 +2340,13 @@ void _wnck_set_window_geometry (Screen *screen,
   xev.xclient.data.l[3] = width;
   xev.xclient.data.l[4] = height;
 
+  _wnck_error_trap_push ();
   XSendEvent (gdk_display,
               RootWindowOfScreen (screen),
               False,
               SubstructureRedirectMask | SubstructureNotifyMask,
               &xev);
+  _wnck_error_trap_push ();
 }
 
 void
@@ -2484,8 +2508,11 @@ static int next_token = 1;
 static void
 _wnck_free_layout_manager (LayoutManager *lm)
 {
+  _wnck_error_trap_push ();
   XDestroyWindow (gdk_display, lm->window);
-  g_free (lm);
+  _wnck_error_trap_pop ();
+
+  g_slice_free (LayoutManager, lm);
 
   layout_managers = g_slist_remove (layout_managers, lm);
 }
@@ -2539,7 +2566,7 @@ _wnck_try_desktop_layout_manager (Screen *xscreen,
 
   /* No one has the selection at the moment */
 
-  lm = g_new0 (LayoutManager, 1);
+  lm = g_slice_new0 (LayoutManager);
 
   lm->screen_number = number;
   lm->token = next_token;
@@ -2548,6 +2575,8 @@ _wnck_try_desktop_layout_manager (Screen *xscreen,
   lm->selection_atom = selection_atom;
   lm->manager_atom = _wnck_atom_get ("MANAGER");
 
+  _wnck_error_trap_push ();
+
   lm->window = XCreateSimpleWindow (gdk_display,
                                     RootWindowOfScreen (xscreen),
                                     0, 0, 10, 10, 0,
@@ -2560,6 +2589,8 @@ _wnck_try_desktop_layout_manager (Screen *xscreen,
   XSetSelectionOwner (gdk_display, lm->selection_atom,
 		      lm->window, timestamp);
 
+  _wnck_error_trap_pop ();
+
   /* Check to see if we managed to claim the selection. */
 
   if (XGetSelectionOwner (gdk_display, lm->selection_atom) !=
@@ -2579,8 +2610,10 @@ _wnck_try_desktop_layout_manager (Screen *xscreen,
   xev.data.l[3] = 0;	/* manager specific data */
   xev.data.l[4] = 0;	/* manager specific data */
   
+  _wnck_error_trap_push ();
   XSendEvent (gdk_display, RootWindow (gdk_display, number),
               False, StructureNotifyMask, (XEvent *)&xev);
+  _wnck_error_trap_pop ();
 
   layout_managers = g_slist_prepend (layout_managers,
                                      lm);
@@ -2607,6 +2640,8 @@ _wnck_release_desktop_layout_manager (Screen *xscreen,
         {
           if (current_token == lm->token)
             {
+              _wnck_error_trap_push ();
+
               /* release selection ownership */
               if (XGetSelectionOwner (gdk_display, lm->selection_atom) !=
                   lm->window)
@@ -2618,6 +2653,8 @@ _wnck_release_desktop_layout_manager (Screen *xscreen,
                                       None, timestamp);
                 }
 
+              _wnck_error_trap_pop ();
+
               _wnck_free_layout_manager (lm);
               return;
             }
